using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win;

namespace ultrawingrid_checkbox_on_header_cs
{
    /// <summary>
    /// Summary description for Form1.
    /// </summary>
    public class Form1 : System.Windows.Forms.Form
    {
        internal Random rnd = new Random();

        #region Constants
        const int _defaultRowCount = 10;
        const int _defaultStringCount = 2;
        const int _defaultIntegerCount = 2;
        const int _defaultDoubleCount = 2;
        const int _defaultDateCount = 2;
        const int _defaultBooleanCount = 2;
        const int _defaultElementCount = 2;
        const int _defaultMinimumRandomInteger = -100;
        const int _defaultMaximumRandomInteger = 100;
        #endregion

        #region GetData Overloaded
        private DataTable GetData(int intRows, int intStrings, int intInts, int intDoubles, int intDates, int intBools, int intEnums)
        {
            #region BuildTable
            DataTable dtReturnValue = new DataTable("Return Value");
            int i, j;
            DataColumn dcTemp;
            DataColumnCollection dccTemp = dtReturnValue.Columns;

            dcTemp = dccTemp.Add("Key", typeof(int));
            dcTemp.AutoIncrement = true;
            dcTemp.ReadOnly = true;

            for (i = 1; i <= intStrings; i++)
                dccTemp.Add("String " + i.ToString(), typeof(String));

            for (i = 1; i <= intInts; i++)
                dccTemp.Add("Integer " + i.ToString(), typeof(int));

            for (i = 1; i <= intDoubles; i++)
                dccTemp.Add("Double " + i.ToString(), typeof(double));

            for (i = 1; i <= intDates; i++)
                dccTemp.Add("Date " + i.ToString(), typeof(DateTime));

            for (i = 1; i <= intBools; i++)
                dccTemp.Add("Boolean " + i.ToString(), typeof(bool));

            for (i = 1; i <= intEnums; i++)
                dccTemp.Add("Elements " + i.ToString(), typeof(Elements));
            #endregion

            #region Populate Table
            DataRow drTemp;

            for (j = 0; j < intRows; j++)
            {
                drTemp = dtReturnValue.NewRow();

                for (i = 1; i <= intStrings; i++)
                    drTemp["String " + i.ToString()] = (1 == i ? new string((char)(65 + ((j - 1) % 26)), 3) : RandomString());

                for (i = 1; i <= intInts; i++)
                    drTemp["Integer " + i.ToString()] = (1 == i ? RandomInteger(0, 100) : RandomInteger());

                for (i = 1; i <= intDoubles; i++)
                    drTemp["Double " + i.ToString()] = RandomDouble();

                for (i = 1; i <= intDates; i++)
                    if (1 == i)
                        drTemp["Date " + i.ToString()] = DBNull.Value;
                    else
                        drTemp["Date " + i.ToString()] = RandomDate();

                for (i = 1; i <= intBools; i++)
                    drTemp["Boolean " + i.ToString()] = RandomBool();

                for (i = 1; i <= intEnums; i++)
                    drTemp["Elements " + i.ToString()] = RandomEnum();

                dtReturnValue.Rows.Add(drTemp);
            }

            dtReturnValue.AcceptChanges();
            #endregion

            return dtReturnValue;
        }

        private DataTable GetData(int intRows, int intStrings, int intInts, int intDoubles, int intDates, int intBools)
        {
            return GetData(intRows, intStrings, intInts, intDoubles, intDates, intBools, _defaultElementCount);
        }

        private DataTable GetData(int intRows, int intStrings, int intInts, int intDoubles, int intDates)
        {
            return GetData(intRows, intStrings, intInts, intDoubles, intDates, _defaultBooleanCount, _defaultElementCount);
        }

        private DataTable GetData(int intRows, int intStrings, int intInts, int intDoubles)
        {
            return GetData(intRows, intStrings, intInts, intDoubles, _defaultDateCount, _defaultBooleanCount, _defaultElementCount);
        }

        private DataTable GetData(int intRows, int intStrings, int intInts)
        {
            return GetData(intRows, intStrings, intInts, _defaultDoubleCount, _defaultDateCount, _defaultBooleanCount, _defaultElementCount);
        }

        private DataTable GetData(int intRows, int intStrings)
        {
            return GetData(intRows, intStrings, _defaultIntegerCount, _defaultDoubleCount, _defaultDateCount, _defaultBooleanCount, _defaultElementCount);
        }

        private DataTable GetData(int intRows)
        {
            return GetData(intRows, _defaultStringCount, _defaultIntegerCount, _defaultDoubleCount, _defaultDateCount, _defaultBooleanCount, _defaultElementCount);
        }

        private DataTable GetData()
        {
            return GetData(_defaultRowCount, _defaultStringCount, _defaultIntegerCount, _defaultDoubleCount, _defaultDateCount, _defaultBooleanCount, _defaultElementCount);
        }
        #endregion

        #region RandomString OverLoaded)
        private string RandomString()
        {
            return RandomString(0);
        }

        private string RandomString(int intLength)
        {
            if (intLength < 1)
                intLength = rnd.Next(1, 51);

            System.Text.StringBuilder sb = new System.Text.StringBuilder(intLength);

            for (int i = 0; i < intLength; i++)
                sb.Append(RandomCharacter());

            return sb.ToString();
        }
        #endregion

        #region RandomCharacter
        private char RandomCharacter()
        {
            char chrReturnValue = (char)rnd.Next(65, 91);

            if (RandomBool())
                return char.ToLower(chrReturnValue);

            return chrReturnValue;
        }
        #endregion

        #region RandomBool
        private bool RandomBool()
        {
            return (rnd.Next(0, 1) == 0);
        }
        #endregion

        #region RandomInteger Overloaded
        private int RandomInteger(int intMin, int intMax)
        {
            return rnd.Next(intMin, intMax + 1);
        }

        private int RandomInteger(int intMin)
        {
            return RandomInteger(intMin, _defaultMaximumRandomInteger);
        }

        private int RandomInteger()
        {
            return RandomInteger(_defaultMinimumRandomInteger, _defaultMaximumRandomInteger);
        }
        #endregion

        #region RandomDouble
        private double RandomDouble()
        {
            return rnd.NextDouble() * 100;
        }
        #endregion

        #region RandomDate
        private DateTime RandomDate()
        {
            DateTime dDate = DateTime.Today;
            dDate = dDate.AddYears(RandomInteger());
            dDate = dDate.AddMonths(RandomInteger());
            dDate = dDate.AddDays(RandomInteger());
            dDate = dDate.AddHours(RandomInteger());
            dDate = dDate.AddMinutes(RandomInteger());
            dDate = dDate.AddSeconds(RandomInteger());

            return dDate;
        }
        #endregion

        #region RandomEnum
        private Elements RandomEnum()
        {
            return (Elements)rnd.Next(0, System.Enum.GetValues(typeof(Elements)).Length);
        }
        #endregion

        internal Infragistics.Win.UltraWinGrid.UltraGrid UltraGrid1;

        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;

        public Form1()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            //
            // TODO: Add any constructor code after InitializeComponent call
            //
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.UltraGrid1 = new Infragistics.Win.UltraWinGrid.UltraGrid();
            ((System.ComponentModel.ISupportInitialize)(this.UltraGrid1)).BeginInit();
            this.SuspendLayout();
            // 
            // UltraGrid1
            // 
            this.UltraGrid1.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                | System.Windows.Forms.AnchorStyles.Left)
                | System.Windows.Forms.AnchorStyles.Right)));
            this.UltraGrid1.DisplayLayout.ViewStyleBand = Infragistics.Win.UltraWinGrid.ViewStyleBand.OutlookGroupBy;
            this.UltraGrid1.Location = new System.Drawing.Point(8, 6);
            this.UltraGrid1.Name = "UltraGrid1";
            this.UltraGrid1.Size = new System.Drawing.Size(612, 314);
            this.UltraGrid1.TabIndex = 6;
            this.UltraGrid1.Text = "UltraGrid1";
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(624, 325);
            this.Controls.Add(this.UltraGrid1);
            this.Name = "Form1";
            this.Text = "UltraWinGrid CheckBox on Header";
            this.Load += new System.EventHandler(this.Form1_Load);
            ((System.ComponentModel.ISupportInitialize)(this.UltraGrid1)).EndInit();
            this.ResumeLayout(false);

        }
        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.Run(new Form1());
        }

        // Create an instance of the CreationFilter. This needs to be Form-Level so 
        // we can catch the event that fires when the CheckBox is clicked. 
        CheckBoxOnHeader_CreationFilter aCheckBoxOnHeader_CreationFilter = new CheckBoxOnHeader_CreationFilter();

        private void Form1_Load(object sender, System.EventArgs e)
        {
            DataSet ds = new DataSet();
            DataTable dt = GetData(100);
            DataTable dt2 = GetData(100);
            dt2.TableName = "Child";
            ds.Tables.Add(dt);
            ds.Tables.Add(dt2);
            ds.Relations.Add(dt.Columns[0], dt2.Columns[0]);
            this.UltraGrid1.DataSource = ds;

            // Tells the WinGrid to use your custom Creation Filter
            this.UltraGrid1.CreationFilter = aCheckBoxOnHeader_CreationFilter;
        }

        /*
         This CreationFilter class will create a CheckBoxUIElement in each Column Header
         whose DataType is boolean. It will fire the HeaderCheckBoxClicked event 
         whenever the CheckBox is clicked.
         Note that in order to maintain the CheckState, this CreationFilter uses the 
         Tag proprty of the Header. So if the program uses the tag for something else
         this will not work. 
        */
        // Implements the CreationFilter interface		
        public class CheckBoxOnHeader_CreationFilter : IUIElementCreationFilter
        {
            // This event will fire when the CheckBox is clicked. 
            public delegate void HeaderCheckBoxClickedHandler(object sender, HeaderCheckBoxEventArgs e);
            public event HeaderCheckBoxClickedHandler _CLICKED;

            public CheckBoxOnHeader_CreationFilter()
            {
                _CLICKED += new HeaderCheckBoxClickedHandler(aCheckBoxOnHeader_CreationFilter_HeaderCheckBoxClicked);
            }

            private void aCheckBoxOnHeader_CreationFilter_HeaderCheckBoxClicked(object sender, CheckBoxOnHeader_CreationFilter.HeaderCheckBoxEventArgs e)
            {
                // Check to see if the column is of type boolean.  If it is, set all the cells in that column to
                // whatever value the header checkbox is.
                if (e.Header.Column.DataType == typeof(bool))
                {
                    foreach (UltraGridRow aRow in e.Rows)
                    {
                        aRow.Cells[e.Header.Column.Index].Value = (e.CurrentCheckState == CheckState.Checked);
                    }
                }
            }

            // EventArgs used for the HeaderCheckBoxClicked event. This event has to pass in the CheckState and the ColumnHeader
            #region HeaderCheckBoxEventArgs
            public class HeaderCheckBoxEventArgs : EventArgs
            {
                private Infragistics.Win.UltraWinGrid.ColumnHeader mvarColumnHeader;
                private CheckState mvarCheckState;
                private RowsCollection mvarRowsCollection;

                public HeaderCheckBoxEventArgs(Infragistics.Win.UltraWinGrid.ColumnHeader hdrColumnHeader, CheckState chkCheckState, RowsCollection Rows)
                {
                    mvarColumnHeader = hdrColumnHeader;
                    mvarCheckState = chkCheckState;
                    mvarRowsCollection = Rows;
                }

                // Expose the rows collection for the specific row island that the header belongs to
                public RowsCollection Rows
                {
                    get
                    {
                        return mvarRowsCollection;
                    }
                }

                public Infragistics.Win.UltraWinGrid.ColumnHeader Header
                {
                    get
                    {
                        return mvarColumnHeader;
                    }
                }

                public CheckState CurrentCheckState
                {
                    get
                    {
                        return mvarCheckState;
                    }
                    set
                    {
                        mvarCheckState = value;
                    }
                }
            }
            #endregion

            private void aCheckBoxUIElement_ElementClick(Object sender, Infragistics.Win.UIElementEventArgs e)
            {
                // Get the CheckBoxUIElement that was clicked
                CheckBoxUIElement aCheckBoxUIElement = (CheckBoxUIElement)e.Element;

                // Get the Header associated with this particular element
                Infragistics.Win.UltraWinGrid.ColumnHeader aColumnHeader = (Infragistics.Win.UltraWinGrid.ColumnHeader)aCheckBoxUIElement.GetAncestor(typeof(HeaderUIElement)).GetContext(typeof(Infragistics.Win.UltraWinGrid.ColumnHeader));

                // Set the Tag on the Header to the new CheckState
                aColumnHeader.Tag = aCheckBoxUIElement.CheckState;

                // So that we can apply various changes only to the relevant Rows collection that the header belongs to
                HeaderUIElement aHeaderUIElement = aCheckBoxUIElement.GetAncestor(typeof(HeaderUIElement)) as HeaderUIElement;
                RowsCollection hRows = aHeaderUIElement.GetContext(typeof(RowsCollection)) as RowsCollection;

                // Raise an event so the programmer can do something when the CheckState changes
                if (_CLICKED != null)
                    _CLICKED(this, new HeaderCheckBoxEventArgs(aColumnHeader, aCheckBoxUIElement.CheckState, hRows));
            }

            public bool BeforeCreateChildElements(Infragistics.Win.UIElement parent)  // Implements Infragistics.Win.IUIElementCreationFilter.BeforeCreateChildElements
            {
                // Don't need to do anything here
                return false;
            }

            public void AfterCreateChildElements(Infragistics.Win.UIElement parent) // Implements Infragistics.Win.IUIElementCreationFilter.AfterCreateChildElements
            {
                // Check for the HeaderUIElement
                if (parent is HeaderUIElement)
                {
                    // Get the HeaderBase object from the HeaderUIElement
                    Infragistics.Win.UltraWinGrid.HeaderBase aHeader = ((HeaderUIElement)parent).Header;

                    // Only put the checkbox into headers whose DataType is boolean
                    if (aHeader.Column.DataType == typeof(bool))
                    {
                        TextUIElement aTextUIElement;
                        CheckBoxUIElement aCheckBoxUIElement = (CheckBoxUIElement)parent.GetDescendant(typeof(CheckBoxUIElement));

                        // Since the grid sometimes re-uses UIElements, we need to check to make sure 
                        // the header does not already have a CheckBoxUIElement attached to it.
                        // If it does, we just get a reference to the existing CheckBoxUIElement,
                        // and reset its properties.
                        if (aCheckBoxUIElement == null)
                        {
                            //Create a New CheckBoxUIElement
                            aCheckBoxUIElement = new CheckBoxUIElement(parent);
                            aCheckBoxUIElement.Enabled = false;
                        }

                        // Get the TextUIElement - this is where the text for the 
                        // Header is displayed. We need this so we can push it to the right
                        // in order to make room for the CheckBox
                        aTextUIElement = (TextUIElement)parent.GetDescendant(typeof(TextUIElement));

                        // Sanity check
                        if (aTextUIElement == null)
                            return;

                        // Get the Header and see if the Tag has been set. If the Tag is 
                        // set, we will assume it's the stored CheckState. This has to be
                        // done in order to maintain the CheckState when the grid repaints and
                        // UIElement are destroyed and recreated. 
                        Infragistics.Win.UltraWinGrid.ColumnHeader aColumnHeader =
                            (Infragistics.Win.UltraWinGrid.ColumnHeader)aCheckBoxUIElement.GetAncestor(typeof(HeaderUIElement))
                            .GetContext(typeof(Infragistics.Win.UltraWinGrid.ColumnHeader));

                        if (aColumnHeader.Tag == null)
                            //If the tag was nothing, this is probably the first time this 
                            //Header is being displayed, so default to Indeterminate
                            aColumnHeader.Tag = CheckState.Indeterminate;
                        else
                            aCheckBoxUIElement.CheckState = (CheckState)aColumnHeader.Tag;

                        // Hook the ElementClick of the CheckBoxUIElement
                        //aCheckBoxUIElement.ElementClick += new UIElementEventHandler(aCheckBoxUIElement_ElementClick);

                        // Add the CheckBoxUIElement to the HeaderUIElement
                        parent.ChildElements.Add(aCheckBoxUIElement);

                        // Position the CheckBoxUIElement. The number 3 here is used for 3
                        // pixels of padding between the CheckBox and the edge of the Header.
                        // The CheckBox is shifted down slightly so it is centered in the header.
                        aCheckBoxUIElement.Rect = new Rectangle(parent.Rect.X + 3, parent.Rect.Y + ((parent.Rect.Height - aCheckBoxUIElement.CheckSize.Height) / 2), aCheckBoxUIElement.CheckSize.Width, aCheckBoxUIElement.CheckSize.Height);

                        // Push the TextUIElement to the right a little to make 
                        // room for the CheckBox. 3 pixels of padding are used again. 
                        aTextUIElement.Rect = new Rectangle(aCheckBoxUIElement.Rect.Right + 3, aTextUIElement.Rect.Y, parent.Rect.Width - (aCheckBoxUIElement.Rect.Right - parent.Rect.X), aTextUIElement.Rect.Height);
                    }
                    else
                    {
                        // If the column is not a boolean column, we do not want to have a checkbox in it
                        // Since UIElements can be reused by the grid, there is a chance that one of the
                        // HeaderUIElements that we added a checkbox to for a boolean column header
                        // will be reused in a column that is not boolean.  In this case, we must remove
                        // the checkbox so that it will not appear in an inappropriate column header.
                        CheckBoxUIElement aCheckBoxUIElement = (CheckBoxUIElement)parent.GetDescendant(typeof(CheckBoxUIElement));

                        if (aCheckBoxUIElement != null)
                        {
                            parent.ChildElements.Remove(aCheckBoxUIElement);
                            aCheckBoxUIElement.Dispose();
                        }
                    }
                }
            }
        }
    }

    enum Elements
    {
        Air,
        Earth,
        Fire,
        Water,
        Energy
    };
}